// Copyright 2010 Christopher Smith
// 
// This file is part of smithers.logicpuzzles.
// 
// smithers.logicpuzzles is free software: you can redistribute it and/or
// modify it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or (at your
// option) any later version.
// 
// smithers.logicpuzzles is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.logicpuzzles. If not, see <http://www.gnu.org/licenses/>.

package smithers.logicpuzzles;

/**
 * Interface representing a logic puzzle.
 */
public interface Puzzle {
    
    /**
     * Checks whether this puzzle can be modified.
     * 
     * @return {@code true} iff this puzzle is mutable
     */
    public boolean isMutable();
    
    /**
     * Creates a copy of this puzzle which cannot be modified further. If a
     * mutable instance of {@code Puzzle} is used to construct a {@code State}
     * or {@code Solver}, the constructer should call this method to get an
     * immutable copy. Copies created by this method should return {@code
     * false} to {@link #isMutable()} and throw {@link
     * ImmutablePuzzleException} when a method attempts to modify them.
     * 
     * @return a copy of this puzzle which cannot be modified. This copy should
     *         be an instance of the same class as the object which created it.
     */
    public Puzzle makeImmutableCopy();
    
    /**
     * Generates a {@code String} representing the puzzle. The string returned
     * by this method may or may not be human-readable.
     * 
     * @return a {@code String} representing the puzzle
     */
    public String toString();
    
}
